"""Backward-compatible facade for the renamed DeepthinkEngine."""

from analogai.deepthink.deepthink_engine import DeepthinkEngine


class Deepthinker(DeepthinkEngine):
	"""Facade class to preserve the public API surface."""


DeepThinker = Deepthinker
Thinker = Deepthinker






from typing import Optional

from analogai.deepthink.application.value_objects.user_agent import UserAgent
from analogai.deepthink.infrastructure.gateways.llm.gateway import LlmGateway
from analogai.deepthink.infrastructure.gateways.llm.enums.llm_provider import LlmProvider
from analogai.deepthink.presentation.preprocessing.previous_messages_stack import PreviousMessagesStack
from analogai.deepthink.messaging.factories.pipeline_factory import MessagePipelineFactory
from analogai.deepthink.messaging.question_answering.factories.question_answering_service_factory import (
    QuestionAnsweringServiceFactory,
)
from analogai.deepthink.messaging.question_answering.question_routing_service import (
    QuestionRoutingService,
)
from analogai.deepthink.messaging.factories.command_factory import ChatCommandFactory
from analogai.deepthink.messaging.factories.chat_flow_factory import ChatFlowBuilder
from analogai.deepthink.messaging.factories.user_agent_resolver_factory import UserAgentResolverFactory
from analogai.deepthink.internal.processors.deepthinker_processor import DeepthinkPipelineProcessor
from analogai.deepthink.internal.processors.large_text_processor import SentenceBatcher
from analogai.deepthink.internal.processors.chat_synthesizer import ChatResponseSynthesizer
from analogai.deepthink.messaging.fact.services.fact_context_service import FactContextWindowService
from analogai.deepthink.messaging.fact.factories.fact_factory import FactServiceFactory
from analogai.deepthink.messaging.fact.factories.fact_domain_service_factory import FactDomainServiceFactory
from analogai.deepthink.messaging.fact.services.fact_context_assembler import FactContextBuilder
from analogai.deepthink.messaging.fact.services.fact_turn_recorder import FactTurnTracker
from analogai.deepthink.messaging.question_answering.enums.synthesized_text_type import (
    SynthesizedTextType,
)
from analogai.deepthink.messaging.fact.repositories.infrastructure.fact_repository import (
    InMemoryFactRepository,
    MongoFactRepository,
)


class DeepthinkEngine:
    _DEFAULT_USER_ID = "default-user"
    _DEFAULT_AGENT_ID = "default-agent"

    def __init__(
        self,
        llm_gateway_instance: Optional[LlmGateway] = None,
        llm_provider: Optional[LlmProvider] = None,
        llm_model: Optional[str] = None,
    ):
        from analogai.deepthink.config import (
            MAX_SENTENCES_PER_INPUT,
            ENVIRONMENT,
        )
        from analogai.deepthink.infrastructure.gateways.llm.llm_gateway_provider_factory import (
            LlmGatewayProviderFactory,
        )
        from analogai.deepthink.infrastructure.gateways.llm.llm_factory import LlmGatewayFactory
        from analogai.deepthink.startup.factories.domain_service_factory import DomainServiceFactory
        from analogai.deepthink.startup.extensions_service_factory import ExtensionsServiceFactory
        from analogai.foundation.infrastructure.services.embedding_service import EmbeddingService

        if not llm_model:
            raise ValueError("llm_model must be provided; defaults are disabled")
        if llm_provider is None:
            raise ValueError("llm_provider must be provided; defaults are disabled")

        self._llm_model = llm_model
        self._llm_gateway_type = llm_provider.gateway_type
        self._llm_gateway_instance = llm_gateway_instance
        LlmGatewayProviderFactory.set_default_gateway_type(self._llm_gateway_type)
        LlmGatewayProviderFactory.set_default_model(self._llm_model)
        if self._llm_gateway_instance is not None:
            self._llm_gateway_instance.set_model(self._llm_model)
            LlmGatewayFactory.set_default_gateway_instance(self._llm_gateway_instance)
        else:
            LlmGatewayFactory.set_default_gateway_type(self._llm_gateway_type)

        base_domain_factory = DomainServiceFactory()
        extensions_factory = ExtensionsServiceFactory(base_domain_factory)
        conversation_service = extensions_factory.get_conversation_service()
        embedding_service = EmbeddingService()
        fact_repository = (
            MongoFactRepository()
            if ENVIRONMENT == "production"
            else InMemoryFactRepository()
        )
        fact_service = FactServiceFactory(
            conversation_service=conversation_service,
            embedding_service=embedding_service,
            repository=fact_repository,
        ).create()
        self._domain_service_factory = FactDomainServiceFactory(fact_service)
        fact_context_assembler = FactContextBuilder(fact_service)
        fact_context_service = FactContextWindowService(fact_service, window=5)
        pipeline_factory = MessagePipelineFactory(self._domain_service_factory)
        pipeline = pipeline_factory.create_message_pipeline(
            notion_enrichment_enabled=False,
            llm_gateway_instance=self._llm_gateway_instance,
        )
        qa_factory = QuestionAnsweringServiceFactory(self._domain_service_factory)
        question_answering_service = qa_factory.create(
            llm_gateway_instance=self._llm_gateway_instance,
            fact_context_assembler=fact_context_assembler,
            fact_context_service=fact_context_service,
            fact_service=fact_service,
        )
        router = QuestionRoutingService(question_answering_service)
        chat_synthesizer = ChatResponseSynthesizer()
        command_factory = ChatCommandFactory(
            pipeline=pipeline,
            question_answering_service=question_answering_service,
        )
        remember_command = command_factory.create_remember_command()
        search_command = command_factory.create_search_command()
        background_info_command = command_factory.create_background_info_command()
        chat_flow_factory = ChatFlowBuilder(
            input_router=router,
            remember_command=remember_command,
            search_command=search_command,
        )
        chat_flow = chat_flow_factory.create()
        user_agent_resolver_factory = UserAgentResolverFactory(
            default_user_id=self._DEFAULT_USER_ID,
            default_agent_id=self._DEFAULT_AGENT_ID,
        )
        user_agent_resolver = user_agent_resolver_factory.create()
        large_text_processor = SentenceBatcher(MAX_SENTENCES_PER_INPUT)
        turn_recorder = FactTurnTracker(fact_service)
        self._processor = DeepthinkPipelineProcessor(
            chat_flow=chat_flow,
            remember_command=remember_command,
            search_command=search_command,
            background_info_command=background_info_command,
            user_agent_resolver=user_agent_resolver,
            large_text_processor=large_text_processor,
            fact_service=fact_service,
            turn_recorder=turn_recorder,
            chat_synthesizer=chat_synthesizer,
        )

    def deepthink(
        self,
        input_text: str,
        user_agent: Optional[UserAgent] = None,
        previous_messages_stack: Optional[PreviousMessagesStack] = None,
    ) -> Optional[str]:
        return self._processor.deepthink(
            input_text=input_text,
            user_agent=user_agent,
            previous_messages_stack=previous_messages_stack,
        )

    def chat(
        self,
        input_text: str,
        user_agent: Optional[UserAgent] = None,
        previous_messages_stack: Optional[PreviousMessagesStack] = None,
    ) -> Optional[str]:
        return self._processor.chat(
            input_text=input_text,
            user_agent=user_agent,
            previous_messages_stack=previous_messages_stack,
        )

    def remember(
        self,
        input_text: str,
        user_agent: Optional[UserAgent] = None,
        previous_messages_stack: Optional[PreviousMessagesStack] = None,
    ) -> Optional[str]:
        return self._processor.remember(
            input_text=input_text,
            user_agent=user_agent,
            previous_messages_stack=previous_messages_stack,
        )

    def search(
        self,
        input_text: str,
        user_agent: Optional[UserAgent] = None,
        previous_messages_stack: Optional[PreviousMessagesStack] = None,
        synthesized_text_type: Optional[SynthesizedTextType] = None,
    ) -> Optional[str]:
        return self._processor.search(
            input_text=input_text,
            user_agent=user_agent,
            previous_messages_stack=previous_messages_stack,
            synthesized_text_type=synthesized_text_type,
        )

    def generate_background_info(
        self,
        input_text: str,
        user_agent: Optional[UserAgent] = None,
        previous_messages_stack: Optional[PreviousMessagesStack] = None,
    ) -> str:
        return self._processor.generate_background_info(
            input_text=input_text,
            user_agent=user_agent,
            previous_messages_stack=previous_messages_stack,
        )

    def get_background_info(
        self,
        input_text: str,
        user_agent: Optional[UserAgent] = None,
        previous_messages_stack: Optional[PreviousMessagesStack] = None,
    ) -> str:
        return self._processor.get_background_info(
            input_text=input_text,
            user_agent=user_agent,
            previous_messages_stack=previous_messages_stack,
        )


DeepThinker = DeepthinkEngine
Thinker = DeepthinkEngine



from enum import Enum

from analogai.deepthink.infrastructure.gateways.llm.enums.llm_gateway_type import (
    LlmGatewayType,
)


class LlmProvider(str, Enum):
    XAI = "xai"
    ANTHROPIC = "anthropic"
    OPENAI = "openai"
    AZURE = "azure"
    AZURE_OPENAI = "azure_openai"
    BEDROCK = "bedrock"

    @property
    def gateway_type(self) -> LlmGatewayType:
        mapping = {
            LlmProvider.ANTHROPIC: LlmGatewayType.ANTHROPIC,
            LlmProvider.OPENAI: LlmGatewayType.OPENAI,
            LlmProvider.AZURE: LlmGatewayType.AZURE,
            LlmProvider.AZURE_OPENAI: LlmGatewayType.AZURE_OPENAI,
            LlmProvider.XAI: LlmGatewayType.GROK,
            LlmProvider.BEDROCK: LlmGatewayType.BEDROCK,
        }
        return mapping[self]



import os
from analogai.foundation.common.utils.env_utils import (
    load_envs_in_order,
    parse_bool,
    parse_int,
    resolve_cpu_count,
)

load_envs_in_order()

ENVIRONMENT = os.getenv('ENVIRONMENT', 'development')

if ENVIRONMENT not in ['development', 'production']:
    raise ValueError(f"Environment variable ENVIRONMENT must be one of: development or production")

QUICK_TESTING = parse_bool(os.getenv('QUICK_TESTING', 'false'), False)
FINAL_APPLICATION_TESTING = parse_bool(os.getenv('FINAL_APPLICATION_TESTING', 'false'), False)
if FINAL_APPLICATION_TESTING:
    QUICK_TESTING = False

DEBUG = parse_bool(os.getenv('DEBUG', 'false'), False)

if QUICK_TESTING:
    LLM_PROCESSING_ENABLED = False
    # STATEMENT_DECOMPOSITION_ENABLED = False
    print(f"QUICK_TESTING mode enabled - LLM_PROCESSING_ENABLED={LLM_PROCESSING_ENABLED}")
else:
    LLM_PROCESSING_ENABLED = parse_bool(os.getenv('LLM_PROCESSING_ENABLED', 'true'), True)
    # STATEMENT_DECOMPOSITION_ENABLED = os.getenv('STATEMENT_DECOMPOSITION_ENABLED', 'false').lower() == 'true'
    print(f"QUICK_TESTING mode disabled LLM_PROCESSING_ENABLED={LLM_PROCESSING_ENABLED}")

TIME_SENSITIVE_TRANSFORMATIONS_ENABLED = parse_bool(
    os.getenv('TIME_SENSITIVE_TRANSFORMATIONS_ENABLED', 'false'),
    False,
)

PROBABILITY_THRESHOLD = 0.4

ENRICHMENT_LIMIT = 10


def _default_remember_max_workers() -> int:
    cpu_count = resolve_cpu_count()
    computed = (cpu_count - 2) * 2
    return max(1, computed)


QUESTION_ANSWERING_MAX_BELIEFS = parse_int(
    os.getenv("QUESTION_ANSWERING_MAX_BELIEFS", "12"),
    12,
)

QUESTION_ANSWERING_MAX_ENTITY_DEPTH = parse_int(
    os.getenv("QUESTION_ANSWERING_MAX_ENTITY_DEPTH", "3"),
    3,
)

_max_sentences_per_input = parse_int(os.getenv("MAX_SENTENCES_PER_INPUT", "3"), 3)
MAX_SENTENCES_PER_INPUT = max(1, _max_sentences_per_input)

_remember_max_workers = parse_int(
    os.getenv("REMEMBER_MAX_WORKERS"),
    _default_remember_max_workers(),
)
REMEMBER_MAX_WORKERS = max(1, _remember_max_workers)

MULTITHREAD_REMEMBER_ENABLED = parse_bool(
    os.getenv("MULTITHREAD_REMEMBER_ENABLED"),
    True,
)

DEFAULT_SYNTHESIZED_TEXT_TYPE_VALUE = os.getenv("DEFAULT_SYNTHESIZED_TEXT_TYPE", "detailed")






# STATEMENT_DECOMPOSITION_ENABLED = os.getenv('STATEMENT_DECOMPOSITION_ENABLED', 'true').lower() == 'true'

print(f"Environment is set to {ENVIRONMENT} at Thinker")



import os

from analogai.deepthink.infrastructure.gateways.llm.enums.llm_gateway_type import LlmGatewayType
from analogai.foundation.common.utils.env_utils import (
    load_envs_in_order,
    parse_bool,
    parse_optional_float,
)


load_envs_in_order()

MONGODB_URI = os.getenv("MONGODB_URI", "")
MONGODB_DATABASE = os.getenv("MONGODB_DATABASE", "")

USE_VECTOR_SEARCH = parse_bool(os.getenv("USE_VECTOR_SEARCH"), True)
SIMILARITY_CUTOFF = parse_optional_float(os.getenv("SIMILARITY_CUTOFF"), 0.4)
FACT_SIMILARITY_CUTOFF = parse_optional_float(
    os.getenv("FACT_SIMILARITY_CUTOFF"),
    0.25,
)

OPENAI_API_KEY = os.getenv("OPENAI_API_KEY", "")
ANTHROPIC_API_KEY = os.getenv("ANTHROPIC_API_KEY", "")
EMBEDDING_MODEL_NAME = os.getenv("EMBEDDING_MODEL_NAME", "")

XAI_API_KEY = os.getenv("XAI_API_KEY", "")

AWS_ACCESS_KEY_ID = os.getenv("AWS_ACCESS_KEY_ID", "")
AWS_SECRET_ACCESS_KEY = os.getenv("AWS_SECRET_ACCESS_KEY", "")
AWS_REGION = os.getenv("AWS_REGION", "us-east-1")
BEDROCK_MODEL_ID = os.getenv("BEDROCK_MODEL_ID", "")

AZURE_AI_FOUNDRY_ENDPOINT = os.getenv("AZURE_AI_FOUNDRY_ENDPOINT", "")
AZURE_AI_FOUNDRY_API_KEY = os.getenv("AZURE_AI_FOUNDRY_API_KEY", "")

AZURE_OPENAI_ENDPOINT = os.getenv("AZURE_OPENAI_ENDPOINT", "")
AZURE_OPENAI_API_KEY = os.getenv("AZURE_OPENAI_API_KEY", "")

TRACK_LLM = parse_bool(os.getenv("TRACK_LLM", "true"), True)

OLLAMA_HOST = os.getenv("OLLAMA_HOST", "localhost")
OLLAMA_PORT = os.getenv("OLLAMA_PORT", "11434")
OLLAMA_MODEL_NAME = os.getenv("OLLAMA_MODEL_NAME", "qwen2.5:0.5b-instruct")


def _resolve_llm_gateway_type(value: str) -> LlmGatewayType:
    normalized = (value or "").strip().lower().replace("-", "_")
    mapping = {
        "anthropic": LlmGatewayType.ANTHROPIC,
        "claude": LlmGatewayType.ANTHROPIC,
        "openai": LlmGatewayType.OPENAI,
        "azure": LlmGatewayType.AZURE,
        "azure_openai": LlmGatewayType.AZURE_OPENAI,
        "azureopenai": LlmGatewayType.AZURE_OPENAI,
        "grok": LlmGatewayType.GROK,
        "xai": LlmGatewayType.GROK,
        "grok3": LlmGatewayType.GROK,
        "grok4_fast_non_reasoning": LlmGatewayType.GROK,
        "bedrock": LlmGatewayType.BEDROCK,
    }
    if normalized in mapping:
        return mapping[normalized]
    raise ValueError(
        "Unknown DEFAULT_LLM_GATEWAY_TYPE value: "
        f"'{value}'. Please provide a valid gateway type."
    )


_DEFAULT_LLM_GATEWAY_TYPE_VALUE = os.getenv("DEFAULT_LLM_GATEWAY_TYPE")
DEFAULT_LLM_GATEWAY_TYPE = (
    _resolve_llm_gateway_type(_DEFAULT_LLM_GATEWAY_TYPE_VALUE)
    if _DEFAULT_LLM_GATEWAY_TYPE_VALUE
    else None
)



from __future__ import annotations

from typing import Optional

from analogai.foundation.infrastructure.services.embedding_service import EmbeddingService  # type: ignore[import-not-found]
from analogai.deepthink.infrastructure import config as infra_config
from analogai.deepthink.messaging.fact.repositories.fact_repository import FactRepositoryPort
from analogai.deepthink.messaging.fact.services.fact_service import (
    FactServicePort,
    ConversationBackedFactService,
    InMemoryFactService,
    RepositoryBackedFactService,
)
from analogai.deepthink.messaging.fact.enums.fact_embedding_mode import FactEmbeddingMode


class FactServiceFactory:
    def __init__(
        self,
        conversation_service: Optional[object] = None,
        embedding_service: Optional[EmbeddingService] = None,
        repository: Optional[FactRepositoryPort] = None,
    ):
        self._conversation_service = conversation_service
        self._embedding_service = embedding_service
        self._repository = repository

    def create(self) -> FactServicePort:
        embedding_mode = (
            FactEmbeddingMode.ENABLED
            if infra_config.USE_VECTOR_SEARCH
            else FactEmbeddingMode.DISABLED
        )
        if self._repository is not None:
            return RepositoryBackedFactService(
                self._repository,
                embedding_service=self._embedding_service,
                embedding_mode=embedding_mode,
            )
        if self._conversation_service is None:
            return InMemoryFactService(
                embedding_service=self._embedding_service,
                embedding_mode=embedding_mode,
            )
        return ConversationBackedFactService(
            self._conversation_service,
            embedding_service=self._embedding_service,
            embedding_mode=embedding_mode,
        )
